#!/usr/bin/env python
""" 
$Id: Gps.py,v 1.8 2005/08/24 02:41:18 rayg Exp $
Copyright 2005, University of Wisconsin Space Science & Engineering Center

    This file is part of the AERI ingest software.

    The AERI ingest software is free software; you can redistribute it and/or 
    modify it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



"""

import logging,os,win32,shutil
import Config
from process import process

log = logging.getLogger("GPS")

class Gps:

    def writeAeriPos(self):
        """Starts a process for writing the Config.AERI_POS file using the 
        config in Config.AERIGPS_CFG and waits for it to return.
        """
        gps = process( exe = Config.AERIGPS_EXE,
                       environ = {'AERIGPS_CFG':Config.AERIGPS_CFG,
                                  'AERI_POS':Config.AERI_POS },
                       output_stream = Config.logStream("aerigps.out"),
                       output_prefix = "aerigps" )
        gps()
        # check whether the position file was written to the workspace
        if not os.path.exists(Config.AERI_POS): 
            # then copy it from the config directory
            src = Config.configPath(win32.path_split(Config.AERI_POS)[1])
            log.info("no position file - copying %s to %s" % (src,Config.AERI_POS))
            shutil.copyfile(src,Config.AERI_POS)

def verifyInstall(args):
    """verify that the subsystem has the pieces it needs to operate
    """
    src = Config.configPath(win32.path_split(Config.AERI_POS)[1])
    all = [ src, Config.AERIGPS_EXE, Config.AERIGPS_CFG ]
    for each in all:
        print "checking for %s" % each
        assert( os.path.exists( each ) )

def test_writePos(args):
    """Runs aerigps and writes to Config.AERI_POS.
    args[0] - # of times to run
    FIXME - needs to be tested with working aerigps.exe
    """
    import time
    gps = Gps()
    if not args:
        args = ['10']
    for i in range(int(args[0])):
        log.info("starting aerigps process")
        gps.writeAeriPos()
        
def test_show_output(args):
    """Runs GPS exe and prints output to standard out.
    An aeri.pos.test file is written with output if GPS QoS
    is satisfied.
    """
    if not args:
        args = [Config.AERIGPS_CFG]
    gps = process( exe = Config.AERIGPS_EXE,
                   environ = {'AERIGPS_CFG':args[0],
                              'AERI_POS':'aeri.pos.test' } )
    gps()
                   

if __name__ == '__main__':
    doc = """python Gps.py writepos [num_runs]
        writepos uses the following environmental vars:
            AERIGPS_CFG - specifies the config file
            AERI_POS - name of the file to write
        Note: Config.AERI_POS is not written unless gps QoS is satisfied
    python Gps.py show_output -- dump the output of the exe to standard out
    python Gps.py verify_install -- verify that files needed to operate are present
    """
    from sys import argv
    logging.basicConfig()
    log.setLevel( logging.INFO )
    tests = { 'writepos':test_writePos,
              "verify_install":verifyInstall,
              "show_output":test_show_output
            }

    if len(argv) > 1 and argv[1] in tests.keys():
        tests[argv[1]](argv[2:])
    else:
        print doc
