#!/usr/bin/env python
"""
$Id: Interferometer.py,v 1.10.2.1 2008/05/08 20:48:00 rayg Exp $
Copyright 2005, University of Wisconsin Space Science & Engineering Center

    This file is part of the AERI ingest software.

    The AERI ingest software is free software; you can redistribute it and/or 
    modify it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



"""

import Config, os, win32, shutil,logging
from process import process

log = logging.getLogger("Interferometer")



class acquire_task:
    def __init__(self,proc,fn_log,flog):
        self._proc = proc
        self._fnlog = fn_log
        self._flog = flog
        
    def wait(self):
        self._proc.wait()
        # read the log file and cat it to our log
        try:
            self._flog.write( file(self._fnlog,'rt').read() )
        except:
            log.warning("unable to transcribe ACQ96NT.LOG to log file")    
        try:
            os.unlink(self._fnlog)
        except:
            log.warning("unable to remove old ACQ96NT.LOG")
            
    def kill(self):
        self._proc.kill()

class Interferometer:
    def __init__(self):
        # create log file of ACQ96NT.LOG output
        self._acqlog = Config.logStream('acq96.log')
        
    def startIngestSpectra(self,set_pathname):
        """Fork a process to ingest the a given .SET file pathname, returning a task proxy.
        Generates SPC data in the same directory as the SET file.
        """
        dirname, filename = win32.path_split( set_pathname )
        stem,exten = os.path.splitext( filename )
        acq96 = process( exe = Config.ACQ96_EXE, work_dir = dirname, arguments = ['M', stem, '-c', '/v' ] )
        acq96.start()
        return acquire_task(acq96,win32.path_join(dirname,Config.ACQ96_LOG),self._acqlog)

def test_interfer(args=None, Ifactory=Interferometer):
    """collect data using generic .SET file
    """
    print "--simple interferometer test 1--"
    I = Ifactory()
    zzz, filename = win32.path_split( Config.OP_SET )
    work_op_set = Config.workPath(filename)
    shutil.copyfile( Config.OP_SET, work_op_set )
    task = I.startIngestSpectra( work_op_set )
    print "collecting data"
    task.wait()
    print "done! exiting.."

if __name__=='__main__':
    doc = """
python Interferometer.py interfer
"""

    from sys import argv
    tests = {    'interfer' : test_interfer
                 }

    if len(argv) > 1 and argv[1] in tests.keys():
        tests[argv[1]](argv[2:])
    else:
        print doc
    
