#!/usr/bin/env python

from win32 import path_join
import logging
import os
import re
from stat import *

log = logging.getLogger("Verify")

# do not include CTRL-M, use UNIX/DOS convert functions
CTRL_Z = chr(26)
NO_NO_CHARS = {CTRL_Z:'CTRL-Z'}

# see stat module
RWX_PERMS = S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH 
RO_PERMS = S_IRUSR | S_IRGRP | S_IROTH

def configPath(name):
    return path_join('config', name)
    
def configSetup3bodyPath(name):
    return path_join('config', 'setup', '3body',name)
 
def configSetup4bodyPath(name):
    return path_join('config', 'setup', '4body', name) 
    
def configSetupRS_3Path(name):
    return path_join('config', 'setup', 'RS_3', name)  
    
def configSetupHasPath(name):
    return path_join('config', 'setup', 'has', name)
    
def configSetupLn2Path(name):
    return path_join('config', 'setup', 'ln2', name)
    
def configSetupOpPath(name):
    return path_join('config', 'setup', 'op', name)

def aeriRunPath(name):
    return path_join('aeri_run',name)
    
def aeriRunHresdPath(name):
    return path_join('aeri_run','hresd',name)
    
def aeriOs2exePath(name):
    return path_join('aeri','os2exe',name)
    
def aeriRunWdtPath(name):
    return path_join('aeri_run','wdt',name)

RWX_PATS = ['*.[eE][xX][eE]',
            '*.[sS][hH]',
            '*.[jJ][nN][iI][lL][iI][bB]',
            '*.[dD][lL][lL]',
            '*.[cC][mM][dD]',
            '*.[bB][aA][tT]',
            '*.[sS][cC][pP]',
            '*.[pP][yY]'  ]

BAT_FILES = [ aeriRunPath('wdtdis.bat'), 
              aeriRunPath('DmvCLI.bat'),
              aeriRunPath('Jada.bat'),
              aeriRunPath('MirrorMon.bat'),
              aeriRunPath('ingest.bat'),
              aeriRunPath('stkill.cmd')
            ]

EXE_FILES = [   aeriRunPath('aemircmd.exe'),
                aeriRunPath('aemirgo.exe'),
                aeriRunPath('aemirprg.exe'),
                aeriRunPath('aemirsaf.exe'),
                aeriRunPath('aereqmir.exe'),
                aeriRunPath('aerigps.exe'),
                aeriRunPath('aerimirp.exe'),
                aeriRunPath('aeriqc.exe'),
                aeriRunPath('hatchon.exe'),       
                aeriRunPath('ohwhscan.exe'),
                aeriRunHresdPath('aerihk.exe'),
                aeriRunHresdPath('aerisihk.exe'),
                aeriRunHresdPath('aescnmrk.exe'),
                aeriRunHresdPath('aewrthk.exe'),
                aeriRunHresdPath('hresd.exe'),
                aeriRunHresdPath('hresdd.exe'),
                aeriRunHresdPath('hresview.exe'),
                aeriOs2exePath('aericalv.exe'),
                aeriOs2exePath('aerisum.exe'),
                aeriOs2exePath('bbcal3p.exe'),
                aeriOs2exePath('ffovcmr.exe'),
                aeriOs2exePath('nlapp.exe'),
                aeriOs2exePath('rwaspc.exe'),
                aeriOs2exePath('scndirav.exe'),
                aeriOs2exePath('zfli.exe'),
                aeriRunWdtPath('testport.exe')
            ]
         
# windows library files
LIB_FILES = [ path_join('aeri_run','jada','lib','dmvJNI.dll'),
              path_join('aeri_run','jada','lib','libdmvJNI.jnilib')
            ]

# Scripts
SCRIPT_FILES = [ aeriRunPath('stkill.scp'), aeriRunPath('init.sh') ]

# python files
PY_FILES = [ 
                aeriRunPath('Autonomic.py'),
                aeriRunPath('Command.py'),
                aeriRunPath('Config.py'),
                aeriRunPath('Gps.py'),
                aeriRunPath('Housekeeping.py'),
                aeriRunPath('Ingest.py'),
                aeriRunPath('Interferometer.py'),
                aeriRunPath('Jada.py'),
                aeriRunPath('MessageServer.py'),
                aeriRunPath('Mirror.py'),
                aeriRunPath('Science.py'),
                aeriRunPath('Sequencer.py'),
                aeriRunPath('Status.py'),
                aeriRunPath('Tidy.py'),
                aeriRunPath('Verify.py'),
                aeriRunPath('Watchdog.py'),
                aeriRunPath('hatchd.py'),
                aeriRunPath('logmerge.py'),
                aeriRunPath('mirscd.py'),
                aeriRunPath('process.py'),
                aeriRunPath('scr2xml.py'),
                aeriRunPath('seq_build.py'),
                aeriRunPath('sigmove.py'),
                aeriRunPath('treeParse.py'),
                aeriRunPath('win32.py')
            ]
 
# files that should have -rwx------ permissions
RWX_FILES = BAT_FILES + EXE_FILES + LIB_FILES + SCRIPT_FILES + PY_FILES

# config files files that should reside in config but not it's subdirs
CFG_FILES = [ 
                configPath('ABCVALS.NUM'),
                configPath('CALRES.VAL'),
                configPath('INST.ID'),
                configPath('RBRIDGE.RES'),
                configPath('aemirsaf.cfg'),
                configPath('aeri.pos'),
                configPath('aericalv.sip'),
                configPath('aeriemis.asc'),
                configPath('aerigps.cfg'),
                configPath('aerihk.cfg'),
                configPath('aerisum.sip'),
                configPath('aesitter.scr'),
                configPath('bbctrl.tmp'),
                configPath('channel.map'),
                configPath('disk_min.cfg'),
                configPath('ffovcmr.si1'),
                configPath('ffovcmr.si2'),
                configPath('hatch.cfg'),
                configPath('instruments.cfg'),
                configPath('jada.cfg.template'),
                configPath('log4j.dtd'),
                configPath('logger.xml.template'),
                configPath('mandb'),
                configPath('mirprgm.mot'),
                configPath('mirror.beg'),
                configPath('motheat.cfg'),
                configPath('nlapp.sip'),
                configPath('op.set'),
                configPath('outgoing.lis'),
                configPath('radiance.scr'),
                configPath('rwaspc.sip'),
                configPath('setup'),
                configPath('sumary.scr'),
                configPath('zfli.si1'),
                configPath('zfli.si2')
            ]

# config files in config\\setup directories
CFG_SETUP_FILES = [ fnc(a) for fnc in [ configSetup3bodyPath, 
                                         configSetup4bodyPath, 
                                         configSetupRS_3Path, 
                                         configSetupHasPath, 
                                         configSetupLn2Path, 
                                         configSetupOpPath ] for a in ['ABCVALS.NUM','channel.map','mirror.beg']
                  ]
                  
MISC_FILES = [  
                aeriRunPath('AERI.VERSION'), 
                aeriRunPath('GPL.txt') 
             ]

TXT_FILES = CFG_SETUP_FILES + CFG_FILES + PY_FILES + SCRIPT_FILES + BAT_FILES 

def everything(): return CFG_SETUP_FILES + LIB_FILES + CFG_FILES + PY_FILES + SCRIPT_FILES + BAT_FILES + EXE_FILES + MISC_FILES

def printAll(args, root_dir=''):
    """Prints all the filenames Verify.py knows about.
    """
    root_dir += '\n'
    print root_dir.join( everything() )

# files that should have -r--r--r--
RO_FILES = [ ]
            
def verifyJadaInstall():
    # if Jada.py does not exist poop out
    import Jada
    Jada.verifyInstall()
    
def verifyRO(args=[], root_dir=''):
    """Verifies all the appropriate config files are present,
    have the correct file format and optionally read-only.
    ro -- if True permissons are checked and changed to the 
    permissions specified in the script configuration if nec.
    Ignores files that are not found.
    """
    
    if not args: args = CFG_FILES
    elif not args[:-1]: args = CFG_FILES + args
    
    for fn in args:
        msg = '%s -- failed read only permissions check'
        f_pth = os.path.join(root_dir, fn)
        if os.path.isfile(f_pth) and (os.stat( f_pth )[0] & 0777 != RO_PERMS):
            if args[-1] == 'fix':
                os.chmod(f_pth, RO_PERMS)
                msg = msg + ' ... FIXED'
            log.info(msg % f_pth)
        elif not os.path.isfile(f_pth):
            pass
            #log.info('%s is not a file or does not exist' % fn)

def verifyRWX(args = [], root_dir=''):
    """Verifies all files in Verify.RWX_FILES have the permissions specified
    in the script configuration logging a message and changing if nec. Ignores
    files that are not found.
    """
    
    if not args: args = RWX_FILES
    elif not args[:-1]: args = RWX_FILES + args
    
    for fn in args:
        msg = '%s -- failed read/write/execute permissions check'
        f_pth = os.path.join(root_dir, fn)
        if os.path.isfile(f_pth) and (os.stat( f_pth )[ST_MODE] & 0777 != RWX_PERMS):
            if args[-1] == 'fix':
                os.chmod(f_pth, RWX_PERMS)
                msg = msg + ' ... FIXED'
            log.info(msg % f_pth)
        elif not os.path.isfile(f_pth):
            pass
            #log.info('%s is not a file or does not exist' % fn)

def changeListPerms(files, root_dir='', perms=RWX_PERMS, fix=None):
    for fn in files:
        msg = '%s -- failed read/write/execute permissions check'
        f_pth = os.path.join(root_dir, fn)
        if os.path.isfile(f_pth) and (os.stat( f_pth )[ST_MODE] & 0777 != perms):
            if fix:
                os.chmod(f_pth, perms)
                msg = msg + ' ... FIXED'
            log.info(msg % f_pth)
        elif not os.path.isfile(f_pth):
            pass
            #log.info('%s is not a file or does not exist' % fn)
        
def verifyExists(files, root_dir=''):
    """Verify the given files exists logging an error if not. Returns
    a list of files that do exist.
    """
    rfiles = []
    if not files: files = everything()
    
    for fn in files:
        f_pth = os.path.join(root_dir, fn)
        if not os.path.exists(f_pth):
            log.error("%s -- does not exist!!" % f_pth)
            rfiles.append(f_pth)
    
    return rfiles
    
def _trimTheFat( dirs = ['aeri','aeri_run','config'], excludes = ['jada'], 
                 workDir = '\\', trimDir = 'trimmed', fix = False ):
    """
    Get all the files in the directories provided. If it's a directory, link it to trimDir
    and remove it. If it's a file, link it and remove it.
    """
    
    from os.path import join, isfile, isdir, exists, split, dirname
    from shutil import copyfile, move
    from os import walk
    
    os.chdir(workDir)
    for chkdir in dirs:
        for root, wdirs, files in walk( chkdir ):
            # exclude paths
            for wdir in wdirs:
                wpath = path_join(root,wdir)
                if wdir in excludes:
                    log.info("excluding %s" % path_join(root,wpath))
                    wdirs.remove(wdir)
            
            # process files
            for fn in files:
                fpath = path_join(root,fn)
                if fn.rsplit('.')[-1] != 'pyc' and fpath not in everything():
                    msg = '%s -- unknown file' % fpath
                    if not exists(trimDir):
                        os.mkdir(trimDir)
                    if fix:
                        move(fpath, path_join(trimDir,fn))
                        msg += ' ... Moved to %s' % trimDir
                    log.info( msg )
    
        # remove empty directories
        #for root, wdirs, files in walk( chkdir ):
        #    if len(wdirs) == 0 and len(files) == 0:
        #        os.remove( root )
                    
def trimFiles( args = [] ):
    """WARNING: can really screw stuff up.
    Trims files in the application space that are not specified in the file
    list returned by running 'python Verify.py printAll'.  
    """
    if args:
        if args[-1] == 'fix':
            if len(args) > 1:
                _trimTheFat(excludes = args[:-1], fix = True)
            else:
                _trimTheFat(fix = True)
        else:
            _trimTheFat(excludes = args)
    else:
        _trimTheFat()
    
def verifyTextFormat(args=[], root_dir=''):
    """Verifies the given text files have CR terminated lines converting
    if necessary.  Ignores files that are not found.
    """ 
    
    if not args: args = BAT_FILES
    elif not args[:-1]: args = BAT_FILES + args
    
    # change those that are specified back to DOS
    for fn in args:
        msg = '%s -- does not have DOS text format'
        f_pth = os.path.join(root_dir, fn)
        if os.path.isfile(f_pth) and len( file(f_pth).read() ) and not isDosText(f_pth):
            if args[-1] == 'fix':
                changeTextFormat([f_pth])
                msg = msg + ' ... FIXED'
            log.info(msg % f_pth)
        elif not os.path.isfile(f_pth):
            pass
            #log.info('%s is not a file or does not exist' % fn)

def changeTextFormat(files, eol='\r\n', root_dir=''):
    """Reads file and writes it back with carrige return line feeds stripping
    white space from the end of the line.
    """
    for fn in files:
        f_pth = os.path.join(root_dir, fn)
        if os.path.exists(f_pth):
            fd = open(f_pth,'rt')
            naked_lines = [ x.rstrip() for x in fd.readlines() ]
            fd.close()
            open(f_pth,'wb').write(eol.join(naked_lines))
        else:
            log.warn('%s does not exist' % f_pth)

def checkNoNoChars(args = [], root_dir=''):
    """Checks for characters that do not belong for provided list of files
    optionally fixing them by replacing with the empty string.
    Permissions are modified to make changes and changed back when finished. 
    """
    
    fix = False
    if args == ['fix']:
        fix = True
        args = TXT_FILES
    elif len(args) and args[-1] == 'fix':
        fix = True
        args = args[:-1]
    elif not len(args):
        args = TXT_FILES
   
    for fn in args:
        found = 0
        msg = '%s -- Contains '
        f_pth = os.path.join(root_dir, fn)
        if os.path.exists(f_pth):
            
            try:
                f_str = open(f_pth, 'rb').read()
            except:
                log.error("%s -- could not be opened" % f_pth)
                continue

            for each in NO_NO_CHARS:
                # replace bad chars with empty string
                if each in f_str:
                    found += 1
                    msg += '%s ' % NO_NO_CHARS[each]
                    if fix:
                        f_str = f_str.replace(each, '')
                    
            if found > 0:
                if fix:
                    perms = os.stat(fn)[ST_MODE]
                    os.chmod(f_pth, 0600)
                    open(f_pth, 'wb').write(f_str)
                    os.chmod(f_pth, perms)
                    msg += ' ... FIXED'
                log.warn(msg % f_pth)
        else:
            log.warn('%s does not exist' % f_pth)


def toDosFormat(files, root_dir=''):
    """Change all EOLs to CR/LF
    """
    changeTextFormat(files, eol='\r\n', root_dir='')
            
def toUnixFormat(files, root_dir=''):
    """Change all EOLs to LF
    """
    changeTextFormat(files, eol='\n', root_dir='')    

def verifyInstall(fix = [], root_dir=''):
    """Verifies the existance, file format, and permissions of files in the
    aeri, aeri_run, and config dirs (all files returned by everything).
    """

    if fix: fix = ['fix']
    log.info('CHECKING THAT FILES EXIST')
    missing_files = verifyExists( everything(), root_dir )
    log.info('VERIFYING PROPER FILE FORMAT')
    verifyTextFormat( BAT_FILES + CFG_FILES + CFG_SETUP_FILES + fix, root_dir )
    log.info('CHECKING RWX PERMISSIONS')
    verifyRWX( RWX_FILES + fix, root_dir )
    log.info('CHECKING RO PERMISSIONS')
    verifyRO( CFG_FILES + CFG_SETUP_FILES + fix, root_dir )
    
    
    
def isDosText(path):
    """Check that a file has DOS and not UNIX EOLs
    """
    return '\r\n' in file(path,'rb').read()
    
def changePermissions(args = []):
    """Recursively change the permissions of all files with file patterns as
    from the Python fnmatch module in the list args[2:] in the directory 
    args[0] to args[1].
    """
    from os.path import walk, join, isfile
    from os import chmod
    from fnmatch import fnmatch
    
    ptns = args[2:]
    if len(args) < 3:
        ptns = RWX_PATS
        
    log.info("Changing %s files to octal permissons %s" %( args[2:], args[1] ) )
    for root, dir, fnames in os.walk(args[0]):
        for fname in fnames:
            for pattern in ptns:
                if fnmatch(fname,pattern):
                    os.chmod(join(root,fname), int('%s' % args[1],8))
    log.info("DONE")
    
if __name__ == '__main__':
    doc = """
    python Verify.py printAll
        %s
    python Verify.py toDosFormat < file >*
        %s
    python Verify.py toUnixFormat < file >*
        %s
    python Verify.py checkNoNoChars [ < file >* ] [ fix ]
        %s
    python Verify.py rwx_perms [ < file >* ] [ fix ]
        %s
    python Verify.py ro_perms [ < file >* ] [ fix ]
        %s
    python Verify.py install [ fix ]
        %s
    python Verify.py trimFiles [ < exclude >* ] [ fix ]
        %s
    python Verify.py changePerms < dir > < octal perms > [< patterns >]
        %s
    """ % (printAll.__doc__,toDosFormat.__doc__,
           toUnixFormat.__doc__, checkNoNoChars.__doc__,verifyRWX.__doc__, verifyRO.__doc__,
           verifyInstall.__doc__,trimFiles.__doc__,changePermissions.__doc__)
           
    from optparse import OptionParser
    from sys import argv, stdout
    
    fmtr = logging.Formatter('%(message)s')
    hndlr = logging.StreamHandler(stdout)
    hndlr.setFormatter(fmtr)
    log.addHandler(hndlr)
    log.setLevel(logging.DEBUG)

    opts = OptionParser(usage=('%prog [options] \n' + doc))
    opts.add_option('-r', '--root-dir',
                    default='/cygdrive/c', dest='root_dir',
                    help="Directory from which to perform all path construction."+
                         "Default: /cygdrive/c")

    options, args = opts.parse_args()

    tests = {
                'printAll':printAll,
                'toDosFormat':toDosFormat,
                'toUnixFormat':toUnixFormat,
                'rwx_perms':verifyRWX,
                'checkNoNoChars':checkNoNoChars,
                'ro_perms':verifyRO,
                'install':verifyInstall,
                'trimFiles':trimFiles,
                'changePerms':changePermissions
            }

    if len(args) and args[0] in tests.keys():
        tests[args[0]](args[1:], root_dir=options.root_dir)
    else:
        print doc

else:
    logging.basicConfig()
