#!/usr/bin/env python
"""
$Id: Watchdog.py,v 1.9 2005/08/24 02:41:18 rayg Exp $
Copyright 2005, University of Wisconsin Space Science & Engineering Center

    This file is part of the AERI ingest software.

    The AERI ingest software is free software; you can redistribute it and/or 
    modify it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



"""

import Config, logging, sys
from process import process
from Tidy import Tidy

log = logging.getLogger('WatchDog')

# program which gives us direct I/O port access
IOEXE = Config.TESTPORT_EXE

def testport_input():
    TESTPORT_INPUT = [
        'W 2E 2 87',
        'W 2E 2 87',
        'W 2E 2 07',
        'W 2F 2 08',
        'W 2E 2 30',
        'W 2F 2 01',
        'W 2E 2 F5',
        'W 2F 2 08',
        'W 2E 2 F6',
        'W 2F 2 %(minutes)02x',
        'exit'
        ]
    return TESTPORT_INPUT

def wdt_process( minutes ):
    wdt_init = process( exe=IOEXE,
                        input_lines= [f % locals() for f in testport_input()],
                        output_stream=Config.logStream('wdt.out'),
                        output_prefix='wdt'
			)
    return wdt_init

class Watchdog:
    def __init__(self):
        wdt = wdt_process( Config.DELAY )
        wdt()

    def reset(self):
        log.info("resetting WatchDogTimer")
        wdt = wdt_process( Config.DELAY ) 
        wdt()

def test_reset( args ):
    """Reset the WDT to a given number of minutes
    """
    minutes = int(args[0])
    # output file goes to a default place
    wdt = wdt_process( minutes )
    wdt()

def test_stop( args ):    
    """ stop the WDT
    """
    test_reset( [ 0 ] )

if __name__=='__main__':
    doc="""$Id: Watchdog.py,v 1.9 2005/08/24 02:41:18 rayg Exp $
    python Watchdog.py reset NMINUTES -- reset the WDT to approximately NMINUTES minutes
    python Watchdog.py stop -- stop the WDT
"""
    argv = sys.argv
    logging.basicConfig()
    log.setLevel( logging.DEBUG )
        
    tests = { 'reset': test_reset,
              'stop': test_stop
              }

    if len(argv) > 1 and argv[1] in tests.keys():
        tests[argv[1]](argv[2:])
    else:
        print doc
