#!/usr/bin/env python 
"""
$Id: hatchd.py,v 1.7 2005/08/24 02:41:18 rayg Exp $
Copyright 2005, University of Wisconsin Space Science & Engineering Center

    This file is part of the AERI ingest software.

    The AERI ingest software is free software; you can redistribute it and/or 
    modify it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


AERI hatch daemon

Environment variables: 
Config.HATCH_STA : state file containing enable/disable state between boots
HATCH_PORT : name of com port to open/close to enable hatch

"""

import os,logging
import Config
from process import process

log = logging.getLogger('hatchd')
        
HATCH_PORT = os.environ[ 'HATCH_PORT' ]
        
# info on the helper
helper_pid = None
helper_pipe = None

def boolish(nfo):
    if 'en' in nfo: return True
    elif 'dis' in nfo: return False
    print "what does '%s' mean?" % nfo
    return None
        
def setState(s):
    print "state is %s" % s
    global helper_pipe
    if s==None: 
        log.warning("bad hatch status file! assuming enable")
        s = True
    Stable = { True: '1\n', False: '0\n' }
    print >>helper_pipe, Stable[s]
    helper_pipe.flush()
    file(Config.HATCH_STA,'wt').write( s and 'enable\n' or 'disable\n' )

def getStateFromFile():
    try:
        nfo = file(Config.HATCH_STA,'rt').read()
        tf = boolish(nfo)
    except:
        log.warning("no hatch status file! assuming enable")
        tf = True
    setState(tf)

def readNewState( nfo ):
    """read client socket with new state information
    """
    print "received '%s'" % nfo
    tf = boolish(nfo)
    setState(tf)
    
def spawnHelper():
    """ Since we don't seem to be able to do simple things like opening and closing
    com ports in windows, we use a helper app to do it for us. Bleh.
    """
    rfd,wfd = os.pipe()
    pid = os.fork()
    if pid==0: # then am child
        os.fdopen(wfd,'w').close()
        os.dup2(rfd,0) # set stdin
        os.execv( Config.HATCHON_EXE, [ Config.HATCHON_EXE, HATCH_PORT ] ) 
        # child python process is now replaced with C++ executable
    os.fdopen(rfd,'r').close()
    wfp = os.fdopen(wfd, 'w')
    return pid, wfp
    
def main( ):
    """Use helper application hatchon.exe to control the DTR of a serial port.
    Allow persistent state.
    Allow state to by changed through socket requests.
    """
    print "spawning helper"
    pid,wfp = spawnHelper()
    global helper_pid, helper_pipe
    helper_pid = pid
    helper_pipe = wfp
    
    getStateFromFile()
    
    # note that we need to fork before importing socket due to gotcha with win32 fork
    # this is due to DLL image relocation misalignment, apparently
    # see notes on Cygwin 'rebase' package
    
    print 'server starting'
    from select import select
    import socket
    srvskt = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
    srvskt.bind( ('127.0.0.1',int(os.environ.get("PORT",'4444'))) )
    srvskt.setblocking(0)
    srvskt.listen(1)
    cliskt = None
    
    while True:
        readlist = [ srvskt ]
        if cliskt: readlist.append( cliskt ) # don't really need this unless client talks to us
        readable, writable, errored = select( readlist, [], [] )
        
        if srvskt in readable: # then connect to a new client, dumping the old one
            print "server connecting to new client"
            if cliskt: 
                print "closing old client connection first"
                cliskt = None
            cliskt,addr = srvskt.accept()
            
        if cliskt in readable:
            nfo = cliskt.recv(1024)
            if nfo:
                readNewState( nfo )
            else:
                cliskt.close()
                cliskt = None
            
if __name__=='__main__':
    import signal
    try:
        main()
    except Exception, ex:
        print ex
        print "killing helper app" 
        os.kill( helper_pid, signal.SIGKILL )
    
