#!/usr/bin/env python 
"""
$Id: logmerge.py,v 1.3 2005/08/24 02:41:18 rayg Exp $
Copyright 2005, University of Wisconsin Space Science & Engineering Center

    This file is part of the AERI ingest software.

    The AERI ingest software is free software; you can redistribute it and/or 
    modify it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


logmerge.py : merge LOG files and OUT files into one time-ordered stream

"""

import time,re

re_hms = re.compile( '(\d\d?):?(\d?\d):?(\d\d)' )

def hms2s(hms):
    tee = re_hms.match(hms).groups()
    h,m,s = [ int(x) for x in tee ]
    #h,m,s = [ int(x) for x in hms.split(':') ]
    return h*3600 + m*60 + s
    
def s2hms(s):
    ss = s%60
    mm = (s/60)%60
    hh = s/3600
    return '%d:%02d:%02d' % (hh,mm,ss)

def gawkstamp( name, ymd, hms ):
    ymd = int(ymd)
    y = ymd/10000
    if y<100: y+=2000
    m = (ymd/100) % 100
    d = ymd % 100
    return hms2s(hms)
    
def pystamp( name, hms, type ):
    return hms2s(hms)
    

STAMP_XFORMS = [ ( re.compile( r'\[(\w+) (\d+) ([:0-9]+)\]\s?' ), gawkstamp ),
                 ( re.compile( r'\[(\w+) ([:0-9]+) (\w+)\]\s?' ), pystamp ) 
                 ]

BIGTIME = 100000000000000000000000L  # FIXME

def extractTime(line):
    for rex,fun in STAMP_XFORMS:
        m = rex.match( line )
        if m: return fun( *m.groups() )
    return None
                 
def merge( mintime=0, maxtime=BIGTIME, *fnlist ):
    from sys import stderr,stdout
    fps = [ file(fn,'rt') for fn in fnlist ]
    lines = [ (fp,fp.readline()) for fp in fps ]
    tfl = [ [extractTime(x), fp, x] for fp,x in lines ]
    now = None
    next = min(tfl)
    dot = mintime - (mintime % 60)
    while now < maxtime:
        if now>=dot:
            fout = stdout
            end = '\n'
            if now < mintime: 
                fout = stderr
                end = '\r'
            print >>fout, '='*40 + ' ' + s2hms(dot) + end,
            dot += 60
        if now >= mintime:
            print next[2],
        li = next[1].readline()
        if not li: 
            next[0] = BIGTIME
            next[1] = None
            next[2] = ''
        else:
            ti = extractTime(li)
            next[0] = ti
            next[2] = li
        next = min(tfl)
        now = next[0]
        
USAGE = """$Id: logmerge.py,v 1.3 2005/08/24 02:41:18 rayg Exp $
Merges time-stamped AERI output and log files for perusal
logmerge.py startHHMMSS endHHMMSS list-of-files
HHMMSS can be '-' for no-limit
Examples: 
logmerge.py - - *.out ingest.log | less
logmerge.py 124007 124100 *.out ingest.log | less
"""

if __name__=='__main__':
    from sys import argv
    if len(argv)<4: print USAGE
    else: 
        txstart,txend = argv[1], argv[2]
        tstart = 0
        tend = BIGTIME
        if txstart!='-': tstart = hms2s(txstart)
        if txend!='-': tend = hms2s(txend)
        merge( tstart, tend, *argv[3:] )
    
    