#!/usr/bin/env python
"""
$Id: mirscd.py,v 1.2.2.2 2008/09/25 20:36:31 rayg Exp $
Copyright 2005, University of Wisconsin Space Science & Engineering Center

Mirror schedule read/write/manipulate

    This file is part of the AERI ingest software.

    The AERI ingest software is free software; you can redistribute it and/or 
    modify it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



"""

import re


re_niviews = re.compile( r'\s*(\d+)\s+(\d+)' )
re_aview = re.compile( r'\s*([-\w]+)\s+(\d+)\s+([.0-9]*)' )


class mirror_schedule:
    """Mirror schedule structure with read-write capability.
    """
    class mirror_schedule_element:
        def __repr__(self):
            return "[position %(name)s, scans %(scans)d, angle %(angle)s, command %(command)s]" % self.__dict__

    def read(self, fp):
        l = fp.readline()
        nviews, iview = re_niviews.findall( l )[0]
        self.nviews = int(nviews)
        self.iview = int(iview) #NOTE this is an index, not an offset
        view = []
        for dex in range(0,self.nviews):
            lmnt = mirror_schedule.mirror_schedule_element()
            lmnt.command = fp.readline().strip()
            if not lmnt.command:
                lmnt.command = None
            name, scans, angle = re_aview.findall( fp.readline() )[0]
            lmnt.name = name.upper()
            lmnt.scans = int(scans)
            if angle: 
                lmnt.angle = float(angle)
            else: 
                lmnt.angle = None
            view.append(lmnt)
        self.views = view
        
    def computeOffset(self):
        # find the first entry in the schedule having both an angle and a command
        ref_positions = [ (x.angle,x.command) for x in self.views if ((x.angle != None) and (x.command != None)) ]
        # number of steps to get to ref_angle
        ref_angle,ref_command = ref_positions[0]
        mot_pos_count = int( ref_command[1:] )
        self.ref_angle = ref_angle
        self.ref_count = mot_pos_count
        
        # find the zero step and location of the hard stop
        offset_count = mot_pos_count - 100*ref_angle
        hard_stop = offset_count + 6300 
        while offset_count < 0: 
            offset_count += 36000
        while hard_stop<0: 
            hard_stop += 36000
        self.offset_count = offset_count
        self.hard_stop = hard_stop
        return self.offset_count, self.hard_stop

    def angleForSteps(self,steps):
        """ given the command steps, find the angle
        """
        if not hasattr(self,'hard_stop'):
            self.computeOffset()
        angle = (steps - self.offset_count) / 100
        while angle >= 360: 
            angle -= 360
        while angle < 0: 
            angle += 360
        return angle

    def stepsForAngle(self,angle):
        """given the angle, find the command steps
        """
        if not hasattr(self,'hard_stop'): 
            self.computeOffset()
        steps = angle*100 + self.offset_count
        while steps < 0: 
            steps += 36000
        while steps > self.hard_stop: 
            steps -= 36000
        return steps
        
    def write(self, fp):
        print >>fp, '%d %d' % (self.nviews, self.iview)
        for view in self.views:
            print >>fp, view.command
            print >>fp, "%s %d %.3f" % (view.name,view.scans,view.angle)

    def __repr__(self):
        txt = ', '.join( [ `x` for x in self.views ] )
        return "[%d views, next is %d: %s]" % (self.nviews, self.iview, txt)
            
    def __init__(self,fp = None):
        if fp: self.read(fp)

