#!/usr/bin/env python
"""
$Id: seq_build.py,v 1.3 2005/08/24 02:41:18 rayg Exp $
Copyright 2005, University of Wisconsin Space Science & Engineering Center

    This file is part of the AERI ingest software.

    The AERI ingest software is free software; you can redistribute it and/or 
    modify it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


This program finds a reference command + angle pair in a mirror schedule file.
Taking the first pair it finds in the file, it then updates missing steps or angles, and writes out a new file.

"""

USAGE = """
Usage: python seq_build.py mirror_input.beg mirror_output.beg
"""


import logging

# mirror schedule reading and writing
from mirscd import mirror_schedule 


def main(args):
    ms = mirror_schedule()
    
    # read the mirror schedule file we're going to fix up
    ms.read( file(args[0],'rt') )
    
    print "input mirror schedule--"
    print ms
    print '-'*40

    ms.computeOffset()
    print "reference position: angle %d at %d steps" % (ms.ref_angle,ms.ref_count)
    print "hard stop is at %d steps; offset is %d steps" % (ms.hard_stop,ms.offset_count)    
    
    print '-'*40
    for pos in ms.views: 
        if None==pos.command:    
            # given the angle, find the command steps
            steps = ms.stepsForAngle(pos.angle)
            pos.command = 'D%d' % steps
            print "computed steps for angle %d: %d" % (pos.angle,steps)
            
        elif None==pos.angle: 
            # given the command steps, find the angle
            steps = int(pos.command[1:])
            pos.angle = ms.angleForSteps(steps)
            print "computed angle for %d steps: %d" % (steps,pos.angle)
    
    print '-'*40
    print "output mirror schedule:"
    print ms
    print "writing output file %s" % args[1]
    ms.write( file(args[1],'wt') )
    
    for ang_str in args[2:]:
        ang = int(ang_str)
        print "steps for angle %d: %d" % (ang, ms.stepsForAngle(ang))
    
if __name__=='__main__':
    from sys import argv
    if len(argv)<3:
        print USAGE
    else:
        main( argv[1:] )
