"""
$Id: win32.py,v 1.12 2005/08/24 02:41:18 rayg Exp $
Copyright 2005, University of Wisconsin Space Science & Engineering Center

    This file is part of the AERI ingest software.

    The AERI ingest software is free software; you can redistribute it and/or 
    modify it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


Compatibility routines for making win32 cygwin python use windows-style paths instead of cygwin-style.

"""

# if not win32, set these routines to behave as os.path.join and os.path.split
# primarily used for testing

import re,sys,os

# separators we want to override for different platforms
_platsep = { 'cygwin': '\\' }

SEP = _platsep.get( sys.platform, os.path.sep )

# allow path separators of all kinds to be used when splitting
RESEP=r'\\|/'

def path_split( pathname ):
    """Split a windows pathname and return a 
       directory name filename pair 
    """
    s = re.split( RESEP, pathname )
    filename = s[-1]
    dirname = SEP.join( s[0:-1] )
    return dirname, filename
    
def path_explode( pathname ):
    return re.split(RESEP,pathname)

def path_join( dirname, *filenames ):
    """ return file pathname
    """
    # dir = dirn.split(SEP)
    s = re.split( RESEP, dirname )
    n = len(s)
    if s[-1]=='': n-=1
    return SEP.join( s[0:n] + list(filenames) )

def unix_path(path):
    """ convert c:\\path to /cygdrive/c/path
    Only works in cygwin!
    """
    if sys.platform!='cygwin': return path
    w,r = os.popen2( ['cygpath', '-u', path], 'rt' )
    return r.read().strip()

def windows_path(path):
    """ convert /cygdrive/c/path to c:\\path
    Only works in cygwin!
    """
    if sys.platform!='cygwin': return path
    w,r = os.popen2( ['cygpath', '-w', path], 'rt' )
    return r.read().strip()
    
